/*
 *  Helper functions for outputting text.
 *  Copyright © 2011, 2021, 2023 Nick Bowler
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <config.h>
#include <stdio.h>
#include <assert.h>

#include "cdecl.h"
#include "cdecl-internal.h"

#include "specstr.h"

size_t cdecl__advance(char **buf, size_t *n, size_t amount)
{
	if (amount >= *n) {
		*n   = 0;
		*buf = 0;
	} else {
		*buf += amount;
		*n   -= amount;
	}

	return amount;
}

size_t cdecl__emit(char **dst, size_t *dstlen, const char *src)
{
	size_t rc = snprintf(*dst, *dstlen, "%s", src);
	return cdecl__advance(dst, dstlen, rc);
}

static size_t explain_spec(char **dst, size_t *dstlen, struct cdecl_declspec *s)
{
	size_t ret;

	ret = cdecl__emit(dst, dstlen, spec_string(s->type));
	if (s->ident) {
		ret += cdecl__emit(dst, dstlen, " " + !ret);
		ret += cdecl__emit(dst, dstlen, s->ident);
	}

	return ret;
}

/*
 * Render a list of declaration specifiers.  Only the declaration specifiers
 * listed in mask, which is the bitwise OR of the desired specifier kinds, are
 * printed.
 */
size_t cdecl__emit_specs(char **dst, size_t *dstlen,
                         struct cdecl_declspec *s,
                         unsigned mask)
{
	size_t ret = 0;

	for (; s; s = s->next) {
		if (!(s->type & mask))
			continue;

		ret += cdecl__emit(dst, dstlen, " " + !ret);
		ret += explain_spec(dst, dstlen, s);
	}

	return ret;
}
